#pragma once

#include <tchar.h>
#include <windows.h>

class ComError
{
public:
	DWORD err;
	ComError(DWORD e):err(e) {}
};

class Com
{
private:
	HANDLE hcom;
public:
	Com():hcom(INVALID_HANDLE_VALUE) {}
	~Com()
	{
		try
		{
			close();
		}
		catch(...){}
	}

	void open(const _TCHAR *name)
	{
		close();
		hcom=CreateFile(name, GENERIC_READ|GENERIC_WRITE, 0, 0,
		OPEN_EXISTING, 0, 0);
		if(hcom==INVALID_HANDLE_VALUE)
			throw  ComError(GetLastError());
	}

	void close()
	{
		if(hcom==INVALID_HANDLE_VALUE)
			return;

		if(CloseHandle(hcom)==0)
			throw  ComError(GetLastError());

		hcom=INVALID_HANDLE_VALUE;
	}

	void config(DWORD baud_rate, BYTE byte_size=8, BYTE parity=NOPARITY, BYTE stop_bits=ONESTOPBIT)
	{
		DCB dcb;
		memset(&dcb, 0, sizeof dcb);
		dcb.DCBlength=sizeof dcb;

		dcb.BaudRate=baud_rate;
		dcb.fBinary=1;
		dcb.fParity=parity!=NOPARITY;

		// DTR: enable = . 0 = +V
		dcb.fDtrControl=DTR_CONTROL_ENABLE;

		// RTS: disable = . 1 = -V
		dcb.fRtsControl=RTS_CONTROL_DISABLE;

		dcb.ByteSize=byte_size;
		dcb.Parity=parity;
		dcb.StopBits=stop_bits;

		if(SetCommState(hcom, &dcb)==0)
			throw  ComError(GetLastError());
	}

	void get_timeouts(COMMTIMEOUTS &timeouts)
	{
		if(GetCommTimeouts(hcom, &timeouts)==0)
			throw  ComError(GetLastError());
	}

	void set_timeouts(COMMTIMEOUTS &timeouts)
	{
		if(SetCommTimeouts(hcom, &timeouts)==0)
			throw  ComError(GetLastError());
	}

	DWORD read(char *buf, DWORD size)
	{
		if(size==0)
			return 0;
		DWORD numofbytes=0;
		if(ReadFile(hcom, buf, size, &numofbytes, 0)==0)
			throw  ComError(GetLastError());
		return numofbytes;
	}

	DWORD write(const char *buf, DWORD size)
	{
		if(size==0)
			return 0;
		DWORD numofbytes=0;
		if(WriteFile(hcom, buf, size, &numofbytes, 0)==0)
			throw  ComError(GetLastError());
		return numofbytes;
	}
};